/*****
 * Station Identification Timer, set for a 10 minute countdown.
 * 
 * Version 1.0, July 20, 2013
 * Dr. Purdum
 * 
 * CAUTION: As presented here, the timer triggers after 10 minutes, which 			essentially means you are in violation of 97.119(a)
 * 
 *****/

#include <LiquidCrystal.h>
// initialize the library with the numbers of the LCD interface pins
LiquidCrystal lcd(12, 11, 7, 6, 5, 4);

unsigned long current = 0;   // Holds the current millisecond count
unsigned long previous = 0;  // Holds the previous millisecond count
int i;
int minutes = 10;            // Set for ten minutes and …
int seconds = 0;             // ...no seconds
char buffer[10];             // Working buffer
void setup() {
  // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);
  					// Print setup message to the LCD.
  lcd.print("Ten Minute Timer");
  lcd.setCursor(5,1);
  sprintf(buffer, "%d:%02d", minutes, seconds); // Stuff 10 minutes and no seconds
                                                // into the buffer
  lcd.print(buffer);  // Show the buffer
}
void loop() {
  current = millis();               // Read the millisecond count
  if (current - previous > 1000) {  // If a second has passed...
    if (previous == 0){             // ...and if first pass through this code...
      lcd.setCursor(5,1);           // ...blank out the "1" character for the "10"
      lcd.print(" ");
    }

    if (seconds == 0 && minutes > 0) {  // If seconds = 0 but minutes are left...
      seconds = 59;                   // ...reset the seconds...
      minutes--;                      // ...decrement the minutes
    } else {
      if (seconds) {
        seconds--;
      }
    }

    if (minutes == 0 && seconds == 0) {
      warnID();
    } else {
      lcd.setCursor(6,1);
      sprintf(buffer, "%d:%02d", minutes, seconds);
      lcd.print(buffer);
      previous = current;
    }
  }
}
/*****
	This function alters the user that time has expired on the ID timer.
	Parameter List:
		void
	Return Value:
		void
*****/
void warnID() {
  static bool flag = true;
  if (flag) {
    current = millis();                // Read the millisecond count
    if (current - previous > 500) {  // If a second has passed...
      lcd.display();
      delay(500);
      lcd.setCursor(0, 0);
      lcd.print("    IDENTIFY    ");
      lcd.setCursor(0,1);
      lcd.print("       NOW      ");
      flag = false;
    }
  } else {
    delay(500);
    lcd.noDisplay();
    flag = true;
  }
  previous = current;
}