/* Simple LCD Panel Meter ver1.0 23 December 2013 D.Kidder W6DQ
  Taken from Chapter 3.
 */
 #include <LiquidCrystal.h>

//                                 Function prototypes
float CalculateTheOutput(int analogIn);
void ShowTheOutput(int analogIn, float val);
void BuildTheOutput(float val);

#define DEFAULTPRECISION    3	// default decimal places
#define MAXANALOGVALUE   1000   // Set valid input to 0 - 1000
#define VALUEWIDTH          8   // Numeric field width, include '-', '.'
#define DIODEVOLTAGEDROP  1.2   // Two 1N4814...your mileage may vary
#define DISPLAYDELAY     2000   // Hold display for two seconds

#define UPDATEDELAY       100   // Delay for normal reading
#define ERRORDELAY        400   // Delay if error message displayed
#define STARTUPDELAY     3000   // Enough time to read splash screen

#define BARGRAPHROW         1   // Draw graph on LCD second row
#define BARGRAPHCOL         0   // Start graph in LCD column 0

const int LCDNUMCOLS =     16;  // number of columns on LCD
const int LCDNUMROWS =      2;
const int DLSENSORPIN =    A1;  // define the analog input as A1

char buffer[LCDNUMCOLS + 1];    // Hold the output
float scaleFactor = 10.0;       // scaling factor for the meter 1000 = 100W 
const float MESASUREDDUMMYLOADRESISTANCE = 50.4;  // Put your load resistance here

LiquidCrystal lcd(12, 11, 7, 6, 5, 4); // setup the LCD hardware interface pins

void setup()                          // Step 1: Initialization
{
  analogReference(EXTERNAL);          // set the A/D to use the AREF input
  lcd.begin(LCDNUMCOLS, LCDNUMROWS);  // initialize the LCD

  lcd.print(" K&P Watt Meter");       //sappy stuff, name, version and date
  lcd.setCursor(0,1);
  lcd.print("ver 1.0 11Mar 14");
  delay(STARTUPDELAY);
  lcd.clear();

}

void loop()
{
  int analogInVolts;
  float val;

  analogInVolts = analogRead(DLSENSORPIN);      // Step 2: Data Input
  if (analogInVolts) {
    BuildTheOutput(analogInVolts);              // Step 3: Process the input
    ShowTheOutput(analogInVolts, val);          // Step 4: Display the data
    delay(DISPLAYDELAY);
  }
          // There is no Step 5...just do everything again!
}


/*****
 * This function takes measured rf power and formats it for display on the LCD
 * 
 * Parameter List:
 * int analogIn      the voltage measured on DLSENSORPIN
 * float watts       the calculated watts into the dummy load
 * 
 * Return value:
 * void
 *****/
void ShowTheOutput(int analogIn, float watts)
{
  lcd.setCursor(0,0);                  // Set up the display...
  lcd.print(buffer);

  if (analogIn <= MAXANALOGVALUE) {    // If analog value within scale...
    delay(UPDATEDELAY);    
  } else {                             // ...otherwise complain about it.
    lcd.setCursor(0, 1);
    lcd.print("   Overrange!   ");  
    lcd.noDisplay();                   // flash the display on and off 
    delay(ERRORDELAY);
    lcd.display();
    delay(ERRORDELAY);       
  } 
}
/*****
 * This function takes measured rf power and formats it for display on the LCD
 * 
 * Parameter List:
 * float watts       the calculated watts into the dummy load
 * 
 * Return value:
 * void
 *****/
void BuildTheOutput(int voltsIn)
{
  int i;
  float scaledVolts;
  float RMS;
  float watts;

  scaledVolts = (float (voltsIn)) + DIODEVOLTAGEDROP;    // Adjust for diode vvoltage drop
  scaledVolts = scaledVolts / scaleFactor;               // Scale it to meter
  RMS = scaledVolts / 1.414;                             // Get RMS of voltage
  watts = (RMS * RMS) / MESASUREDDUMMYLOADRESISTANCE;    // watts into DL
  dtostrf(watts, VALUEWIDTH, DEFAULTPRECISION, buffer);  // Convert and format value
  strcat(buffer, " watts");
}

