/******
 * CW keyer using capacitive touch sensors for keying.
 * 
 * Jack Purdum, Oct. 24, 2013
 * 
 *****/
 
//#define DEBUG 1

#define WHATSNEXT   0
#define DIT    1
#define DAH    2
#define SPEEDUPDITS     9    // If we read > 9 consecutive dits, bump the wpm up
#define LOWERSPEEDDAHS  9    // If we read > 9 consecutive dahs, lower the wpm
#define ONEWPM 7            // This is 1200 divided by 13 wpm, so one wpm = 92 / 13

#define DITPADDLEPIN  0      // This is actually pin 5 on the ATtiny85
#define DAHPADDLEPIN  3      //           "          2      "
#define PADDLECOMMON  4      //           "          3      "
#define OUTPUTPIN     2      // This is used to key the transmitter  

#define ONBOARDLED           1    // This only makes sense for Digispark version. 

#define DITMULTIPLIER        1  // The dit is the basic unit of time
#define DAHMULTIPLIER        3  // A dah is three times a dit
#define LETTERSPACE          (DITMULTIPLIER * 2)
#define WORDSPACE            (DITMULTIPLIER * 7)

#define SENSORDELAY          10        // Let the sensor state stabilize


int dit;
int dah; 
int wordsPerMinute;
int state;
int trackDits;
int trackDahs;

void readDit()
{
  digitalWrite(PADDLECOMMON,HIGH);
  delayMicroseconds(SENSORDELAY);
  if (digitalRead(DITPADDLEPIN)) {
    dit=0; 
  } else {
    dit=1;
  }
  digitalWrite(PADDLECOMMON,LOW);  
}

void readDah()
{
  digitalWrite(PADDLECOMMON,HIGH);
  delayMicroseconds(SENSORDELAY);
  if (digitalRead(DAHPADDLEPIN)) {
    dah=0; 
  } else { 
    dah=1;
  }
  digitalWrite(PADDLECOMMON,LOW);  
}

void setup()
{
  pinMode(PADDLECOMMON, OUTPUT);
  pinMode(OUTPUTPIN, OUTPUT);        // Use for keying circuit
  digitalWrite(PADDLECOMMON, LOW);
//  pinMode(LED,OUTPUT);
//  digitalWrite(LED,LOW);   //turn off led
  state = 0;
  wordsPerMinute = 13;          // Default
  dit = 1200 / wordsPerMinute;      
  dah = dit * 3;
#ifdef DEBUG 
Serial.begin(115200);
#endif

}

//handles the output of the ciruit. if state is 1 then the contact is closed or led is turned on
void contact(unsigned char state)
{
  if (state) {
    digitalWrite(OUTPUTPIN, HIGH);      // Turn output on
    //analogWrite(3,127);  //pin 3 drives an 8 Ohm speaker
  } 
  else {
//    digitalWrite(LED,LOW);  
    digitalWrite(OUTPUTPIN, LOW);      // Turn output off
    //analogWrite(3,0);
  }
}

void loop()
{
  if (trackDits > SPEEDUPDITS) {
    RaiseSpeed();
  }
  if (trackDahs > LOWERSPEEDDAHS) {
    LowerSpeed();
  }
#ifdef DEBUG 
Serial.print("State = ");
Serial.println(state);
#endif
  
  switch (state) {
    case WHATSNEXT:
      readDit();
      if(dit) {
        state = DIT;              
      } else {
        delayMicroseconds(SENSORDELAY);
        readDah();
        if(dah) {
          state = DAH;              
        }
      }      
      break;
  
    case DIT:
      contact(1);
      delay(dit);
      contact(0);
      delay(LETTERSPACE);
      //now, if dah is pressed go there, else check for dit
      readDah();
      if (dah){
        state = DAH;  
        trackDahs++;
        trackDits = 0;      
      } else {
        //read dit now
        readDit();
        if (dit) {
          state = DIT;   
          trackDits++;
          trackDahs = 0;          
        } 
        else {
          delay(LETTERSPACE);
          state = WHATSNEXT;
        }
      }        
      break;
  
  
    case DAH:
      contact(1);
      delay(dah);
      contact(0);
      delay(LETTERSPACE);
      //now, if dit is pressed go there, else check for dah
      readDit();
      if (dit){
        state = DIT;        
      } else {
        //read dit now
        readDah();
        if (dah) {
          state = DAH;              
        } else {
          delay(LETTERSPACE);
          state = WHATSNEXT;
        }
      }  
      break;
      
    default:      
      break;
  }

  delay(1);
}

void RaiseSpeed()
{
  dit += ONEWPM;
  dah = dit * 3;
  trackDits = 0;
  trackDahs = 0;
#ifdef DEBUG 
Serial.print("In RaiseSpeed()  dit = ");
Serial.println(dit);
#endif

}

void LowerSpeed()
{
  dit -= ONEWPM;
  dah = dit * 3;
  trackDits = 0;
  trackDahs = 0;
#ifdef DEBUG 
Serial.print("In LowerSpeed() dit = ");
Serial.println(dit);
#endif
}


