#if (ARDUINO < 100)
#include <WProgram.h>
#else
#include <Arduino.h>
#endif


#define DEBUG            1       // Only on when debugging

// ============================== LCD pins =================================
#define LCDDATA4PIN        22     // Data pins for LCD display. Changed to agree with DFRobot Expansion board, Board 1.
#define LCDDATA3PIN        23
#define LCDDATA2PIN        24
#define LCDDATA1PIN        25
#define LCDENABLEPIN       13     // Enable register write
#define LCDRSPIN           29     // Register select 

// =============================== Keyer pins ===============================
#define DITPADDLEPIN       45      // This is actually pin 5 on the ATtiny85
#define DAHPADDLEPIN       47      //           "          2      "
#define OUTPUTPIN          51      // This is used to key the transmitter via pin 1 on optoisolator  
#define DITMULTIPLIER       1      // The dit is the basic unit of time
#define DAHMULTIPLIER       3      // A dah is three times a dit
#define DAHTRIGGER          9
#define DITTRIGGER          9
#define DITADJUST           7      // 92 = 1200/13 for 13 wpm. To speed up to 14 wpm, the value becomes
                                   // 85 = 1200/14. Therefore, raising one wpm changes the dit time by
                                   // -7 milliseconds. Because we want to be able to adjust up or down,
                                   // this is an unsigned number.

// =============================== PS2 Keyer pins ===========================

#define PS2CLOCKPIN         5        // From keyboard
#define PS2DATAPIN          6        //      "
#define PS2OUTPUTPIN       51        // To optoisolator, pin 1

// =============================== RTC pins ================================
#define RESETSWITCHPIN     28     // Used by RTC which is set to Board 1 on the DFRobot Expansion board.
#define DS1307REGISTERBYTES 7     // Number of data bytes stored in the DS1307 control register

#define LEDPIN             13
#define TONEPIN            A3      // Used if you want audio via a small buzzer


// =============================== decoder constants ============================
#define AUDIOPIN           A5     // Pin for audio input
#define LCDCOLUMNS         16     // LCD column count
#define LCDROWS             2     // LCD row count
#define LINEEND            (LCDCOLUMNS + 1)
#define NOMORSEDELAY     3000     // If no signal in 3 seconds, assume none there
#define NOSIGNALMAX       200     // Wait for this number of passes through the loop before saying
                                  // there's no more sending going on.
// =============================== PS2 constants ===============================

#define QUEUESIZE          128
#define QUEUEMASK           (QUEUESIZE-1)

#define DEFAULTWPM	    15	 // MIN_WPM <= DEFAULT_WPM <= MAX_WPM
#define	MINWPM		     5
#define MAXWPM		    50
#define NOCHARAVAILABLE     -1

#define SIDETONEFREQ       700
#define ESCAPEKEY           27
#define PERCENTKEY          37
#define NEWLINE           '\n'

// =============================== CapKeyer constants ===============================
#define ONEWPM               7    // This is 1200 divided by 13 wpm, so one wpm = 92 / 13

#define WHATSNEXT            0
#define DIT                  1
#define DAH                  2
#define SPEEDUPDITS          9    // If we read > 9 consecutive dits, bump the wpm up
#define LOWERSPEEDDAHS       9    // If we read > 9 consecutive dahs, lower the wpm

#define DITMULTIPLIER        1    // The dit is the basic unit of time
#define DAHMULTIPLIER        3    // A dah is three times a dit
#define LETTERSPACE         (DITMULTIPLIER * 2)
#define WORDSPACE           (DITMULTIPLIER * 7)

#define SENSORDELAY         10   // Let the sensor state stabilize


// =============================== RTC constants ===============================

#define RTCI2CADDRESS    0x68   // 7 bit device address (without last bit - look at the datasheet)

#define IDTIMESLICE    600000   // 10 minutes. Change to shorter period for debugging...
#define IDTIMESLICE     10000   // For debugging, comment one above out...
#define HALFSECOND        500
#define DEBOUNCEDELAY      20
#define ERRORREPEAT         6   // The number of times to repeat blinking error LED indicator
#define FLIPDELAY           1   // Alternates LCD asterisk after this many seconds
#define MODESWITCHDELAY  5000   // If they hold the push botton longer than 5 seconds, activeTimer
                                // changes state.
                                 
#define MILLENIA         2000
//#define EXTERNALLEDPIN     30   // This is used to blink an external LED to notify time expiration. Uncomment to use

#define SECONDS             0   // These are offsets into the DS1307 internal registers
#define MINUTES             1
#define HOURS               2
#define DAY                 3   // This is a day-of-the-week index register
#define DATE                4
#define MONTH               5
#define YEAR                6

#define RTCWRITEERROR       1   // Unable to write RTC time and date to DS1307
#define RTCREADERROR        2   // Unable to read RTC time and date from DS1307


