/*****
 This code is the main entry point into the Integration Project 
 discussed in Chapter 11. It combines the following projects:
   1. PS2 keyboard keyer
   2. CW decoder
   3. Station ID timer
 Each of these projects is separated into its own class. This 
 source file coordinates the action contained within those 
 classes
 
 Jack Purdum, W8TEE
 Jan 10, 2014
 *****/
#include <inttypes.h>
#include <Arduino.h>
#include <PS2Keyboard.h>
#include <LiquidCrystal.h>
#include <Wire.h>
#include <RTClib.h>
#include "IntegrationCode.h"
#include "Decoder.h"
#include "PS2Keyer.h"
#include "RTCTimer.h"

//#define DEBUG 1    // Uncomment if you want to add debug print statements

// ====================== Constructors for objects ===========================

LiquidCrystal lcd(LCDRSPIN, LCDENABLEPIN, LCDDATA1PIN, LCDDATA2PIN, LCDDATA3PIN, LCDDATA4PIN);  // For our LCD shield
Decoder myDecoder;
PS2Keyer myPS2Keyer;
RTCTimer myTimer;

int flag = 0;

void setup() {                
#ifdef DEBUG                    // Use when debugging
Serial.begin(115200);
#endif

  pinMode(LEDPIN, OUTPUT);       // The Arduino LED pin 
  pinMode(AUDIOPIN, INPUT);      // Audio input pin for decoder
  digitalWrite(AUDIOPIN, HIGH);  // Set internal pull-up resistor
  lcd.begin(LCDCOLUMNS, LCDROWS);  
  
  myDecoder.DecoderStartup(AUDIOPIN);  // Decoder setup() start
  myPS2Keyer.PS2Startup();     // PS2 Keyer setup() start
  myTimer.RTCStartup();        // RTC setup() start
  lcd.clear();
}

void loop() {
  int i;
  int sentBack;
  int audio;
  
  audio = digitalRead(AUDIOPIN); // What is the tone decoder doing?

  if (audio == 0) {
    lcd.clear();
    myDecoder.DecoderPoll();    // Pulled low when a signal appears
  } else {
    myTimer.RTCPoll();          // Otherwise, just run the RTC
  }
 
  myPS2Keyer.PS2Poll();         // Are they using the PS2 keyer?

#ifdef DEBUG                  // freeRam() gives approximate SRAM left
if (flag == 0) {
  Serial.print("Free SRAM = ");
  Serial.println(freeRam());
  flag++;
}
#endif
}

/*****
  Function that gives an estimate of the amount of SRAM available. It is only an
  approximation because SRAM ebbs and flows as the program executes, so the amount
  of SRAM is only taken at a point in the program, not for the entire execution 
  sequence.
  
  Parameter list:
    void
    
  Return value:
    int            the amount of unused SRAM at this point in the program
    
*****/
int freeRam()
{
  extern int __heap_start, *__brkval;
  int v;
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval);
}
