/******
*
*   Test program to verify that sensors, switch and display are working correctly
*
******/

#include <LiquidCrystal.h>

#define LCDCOLS 16
#define LCDROWS 2
#define FORWARDPOWERINPUT 1
#define REFLECTEDPOWERINPUT 2
#define SWITCH 7

#define DEBOUNCEDELAY   10

#define BLUELIGHT 8 
#define GREENLIGHT 9
#define REDLIGHT 10

LiquidCrystal lcd(12, 11, 5, 4, 3, 2); // For LCD display from Chapter 3

void setup()
{
  
  analogReference(EXTERNAL);     // use ext reference from LM324 voltage follower (~3.7 VDC)

  lcd.begin(LCDROWS, LCDCOLS);   // initialize the LCD
  lcd.clear();
  lcd.home();
  lcd.print("SWR  METER TEST");  //sappy stuff, name, version and date
  lcd.setCursor(0,1);
  lcd.print("ver0.1 01MAR14");
  delay(3000);                   // Delay probably okay since we're not doing anything else
  lcd.clear();
  
  pinMode(REDLIGHT,OUTPUT);
  pinMode(GREENLIGHT,OUTPUT);
  pinMode(BLUELIGHT,OUTPUT);
  
  pinMode(SWITCH,INPUT);
  digitalWrite(SWITCH,HIGH);
    
}

void loop()
{
  
  digitalWrite(REDLIGHT,LOW);
  digitalWrite(GREENLIGHT,LOW);
  digitalWrite(BLUELIGHT,LOW);

  lcd.home();
  lcd.print(analogRead(FORWARDPOWERINPUT));
  lcd.setCursor(0,1);
  lcd.print(analogRead(REFLECTEDPOWERINPUT));
  
  if (debounce(SWITCH) == true){
    
    digitalWrite(REDLIGHT,LOW);
    digitalWrite(GREENLIGHT,HIGH);
    digitalWrite(BLUELIGHT,HIGH);
    delay(1000);
    digitalWrite(REDLIGHT,HIGH);
    
    digitalWrite(GREENLIGHT,LOW);
    delay(1000);
    digitalWrite(GREENLIGHT,HIGH);
    
    digitalWrite(BLUELIGHT,LOW);
    delay(1000);
    digitalWrite(BLUELIGHT,HIGH);

  }
    
}
/*****
  This function is a pretty standard debounce function so we can determine
  that the switch has really been pressed.
  
  Parameter list:
    int pin            the pin that registers the switch state
    
  Return value:
    boolean            true if the switch was pressed, false otherwise
    
*****/
boolean debounce(int pin)
{
  boolean currentState;
  boolean previousState;
  int i; 
  
  previousState = digitalRead(pin);
  for (i = 0; i < DEBOUNCEDELAY; i++) {
    delay(1);    // small delay
    currentState = digitalRead(pin);  // Read it now
    if (currentState != previousState)
    {
      i = 0;
      previousState = currentState;
    }
  }
  if (currentState == LOW)
    return true;
  else
    return false;
}

