/************
*    A Simple Frequency Counter  ver 1.2 6 May 2014 D.Kidder W6DQ and J.Purdum, W9NMT 
*    
*    This frequency counter is based on the work of:
*
*    Martin Nawrath KHM LAB3
*    Kunsthochschule f¸r Medien Kˆln
*    Academy of Media Arts
*    http://www.khm.de
*    http://interface.khm.de/index.php/labor/experimente/
*
*    and uses the FreqCounter.h library from the same source.
*
*    Pulses are counted on Digital Pin 5 during the gate time and then converted into the
*    frequency in Hz, KHz, or MHz, depending on settings in the code used during compile.

*    The counter has a maximum frequency range of a little over 6 MHz without prescaling.
*    By adding a simple divider consisting of one or two D-Type Flip-Flops arranged as
*    divide by 2 or by 4 respectively.  Using divide by 4 will raise the maximum frequency
*    to roughly 24 MHz.
*
*************/

#include <FreqCounter.h>
#include <LiquidCrystal.h>
#include <stdlib.h>

//#define DEBUG

#define DEFAULTPRECISION 6	   // Default decimal places
#define VALUEWIDTH       10        // Numeric field width

#define LCDNUMCOLS  	 16       // number of columns on LCD
#define LCDNUMROWS       2        // number of rows

#define STARTUPDELAY     3000     // Delay used for splash screen at startup
#define UPDATEDELAY      100      // Delay for normal reading

#define CALIBRATION      0        // Adjusts frequency for variation in Arduino clock rate
#define GATETIME         1000      // Gate time in milliseconds
                                  
#define MEGAHERTZ        1000000  // Let's define a couple of constants
#define MILLISECONDS     1000     // Number of milliseconds in a second

/******
*
* The following three definitions are for using different prescaling divider circuits.
* Only one line should ever be uncommented.  The prescaler circuit divides the input
* by 2 or 4 or not at all.  To adjust the displayed frequency, we multiply the result
* by the same amount.
*
******/

#define PRESCALER      1        // Uncomment when no divider is used
//#define PRESCALER      2        // Uncomment when divide by two is used
//#define PRESCALER      4        // Uncomment when divide by four is used


#define FIRSTIF          12.0     //  First IF frequency of the radio
#define CALOFFSET        -.004988  //  Determined through testing ... offset from 1st IF
#define SECONDIF         0.0      //  Second IF frequency of the radio (if used)

char  scale[] = " MHz";           // Scale string for display units
float scaleFactor = 1.0;          // Scale factor used to adjust units.  
                                  // Default is 1.0 for Hz, 0.001 for KHz and .000001 for MHz

LiquidCrystal lcd(12, 11, 9, 8, 7, 6); 	  // Setup the LCD hardware interface pins

void setup()
{

#ifdef DEBUG

  Serial.begin(115200);

#endif

  lcd.begin(LCDNUMCOLS,LCDNUMROWS);      // initialize the LCD

  lcd.print("Freq Counter ");             //sappy stuff, name, version and date
  lcd.setCursor(0,1);
  lcd.print("ver 1.2 27Mar14");
  delay(STARTUPDELAY);
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Frequency:");

}

void loop() {

  char buffer[LCDNUMCOLS + 1];  // Make it big enough for a 2x16 display
  unsigned long dataIn;
  float val; 
  int i;

  FreqCounter::f_comp=CALIBRATION;          // Calibrate with known source
  FreqCounter::start(GATETIME);             // Count pulses for specified gating period
  
  while (FreqCounter::f_ready == 0) 

     dataIn = FreqCounter::f_freq;;
    
  delay(20); 

  val = (float) dataIn * PRESCALER * scaleFactor * ( MILLISECONDS / GATETIME) / MEGAHERTZ ; // Scale the input reading...

#ifdef  DEBUG
  Serial.print(val);
  Serial.print("   ");
#endif

  val = FIRSTIF + CALOFFSET - val;

#ifdef DEBUG
  Serial.println(val);
#endif

  dtostrf(val, VALUEWIDTH, DEFAULTPRECISION, buffer);                       // Convert and format value
  
  strcat(buffer, scale);               // Concatenate contents of buffer and the scale units into buffer
    
  lcd.setCursor(0,1);                  // Set up the display...
  lcd.print(buffer);

}
