#include <EEPROM.h>

#define READCURRENTFREQ    0                     // The EEPROM record address of the last written frequency
#define READUSERCOUNTS     (READCURRENTFREQ + 1) // The EEPROM record address of number of records saved by the user.
#define READLASTBANDFREQ   (READUSERCOUNTS + 1)  // The starting EEPROM record address for the 9 bands
#define READUSERSTOREDFREQ (READLASTBANDFREQ + 9)// The starting EEPROM record address for 10 user-saved frequencies. Note
                                 // that READUSERSTOREDFREQ plus the number stored at READUSERCOUNTS is the
                                 // next empty address.
#define READUPPERBANDEDGES READUSERSTOREDFREQ + 10  // The are the upper band edges for the 9 bands

#define MAXUSERFREQS      10      // The maximum number of frequencies the user can store
#define MAXBANDS          9       // Number of ham bands covered

void setup()
{
  Serial.begin(115200);
  InitEEPROMAFrequencies();
}
void loop()
{
  // Nothing to do here
}

void InitEEPROMAFrequencies()
{
  int i;
  int offset;
  
  union {
    byte array[4];
    unsigned long val;
  } myUnion;
  
  //                            Code for writing QRP frequencies
  
  unsigned long qrp[] = {1818000,3560000,7030000,10116000,14060000, 18096000,21060000,24906000,28060000, 29700000};
 
  for (i = 0; i < MAXUSERFREQS; i++) {
    myUnion.val = qrp[i];
    offset = (READUSERSTOREDFREQ + i) * sizeof(unsigned long);
    EEPROM.write(offset, myUnion.array[0]);
    EEPROM.write(offset + 1, myUnion.array[1]);
    EEPROM.write(offset + 2, myUnion.array[2]);
    EEPROM.write(offset + 3, myUnion.array[3]); 
  }
 Serial.println("==================== QRP Done ================");
 
  //                            Code for writing lower edge band frequencies
  
  unsigned long lowBandEdge[] = {28000000, 24890000,21000000,18068000,14000000,10100000,7000000,3500000,1800000};

  for (i = 0; i < MAXBANDS; i++) {
    myUnion.val = lowBandEdge[i];
    offset = (READLASTBANDFREQ + i) * sizeof(unsigned long);
    EEPROM.write(offset, myUnion.array[0]);
    EEPROM.write(offset + 1, myUnion.array[1]);
    EEPROM.write(offset + 2, myUnion.array[2]);
    EEPROM.write(offset + 3, myUnion.array[3]); 
  }
 Serial.println("==================== Low Edge Done ================");

  
  //                            Code for writing band frequencies upper edges
  
  unsigned long hiBandEdge[] = {29700000, 24990000,21450000,18168000,14350000,10150000,7300000,4000000,2000000};
 
  for (i = 0; i < MAXBANDS; i++) {
    myUnion.val = hiBandEdge[i];
    offset = (READUPPERBANDEDGES + i) * sizeof(unsigned long);
    EEPROM.write(offset, myUnion.array[0]);
    EEPROM.write(offset + 1, myUnion.array[1]);
    EEPROM.write(offset + 2, myUnion.array[2]);
    EEPROM.write(offset + 3, myUnion.array[3]); 
  }
  Serial.println("==================== High Edge Done ================");
}


