/*****

Code based upon:
  http://forum.pololu.com/viewtopic.php?f=15&t=6424

*****/

#define DEBUG 1                  // Used to toggle debug code

#define SETTLESTEPPER 10         // Time for stepper to change state
#define READSENSORDELAY 5        // Time for sensors to stabilize

#define TOPSENSORPIN A0          // Pin for top sensor
#define BOTTOMSENSORPIN  A2      // Pin for bottom sensor

#define SLEEPPIN 11              // Sleep pin on driver
#define RESETPIN 12              // Reset pin     "

#define STEPPIN   9              // Step pin      "
#define DIRECTIONPIN 8           // Direction pin "
#define ENABLEPIN 10             // Enable pin    "

#define SENSORSENSITIVITY 60     // Sensor difference that causes a rotation
#define STEPPERINCREMENT 10	   //  This many steps (18 degrees) to move panel

int directionVal = HIGH;

int topPhotoValue = 0;           // reading from the top photoresistor
int bottomPhotoValue = 0;        //          "      bottom   "
int previousValue = 0;           // Previous reading from sensor

// ============================= setup() ==================
void setup()
{  
#ifdef DEBUG  
  Serial.begin(115200);  
#endif
  pinMode(SLEEPPIN,OUTPUT);          // Initialize stepper pins on uC
  pinMode(RESETPIN,OUTPUT);
  pinMode(DIRECTIONPIN,OUTPUT);
  pinMode(STEPPIN,OUTPUT);
  pinMode(ENABLEPIN, OUTPUT);
  
  pinMode(TOPSENSORPIN, INPUT);      // Set sensor pins
  pinMode(BOTTOMSENSORPIN, INPUT);

  digitalWrite(SLEEPPIN, HIGH);      // Make sure defaults are set
  digitalWrite(RESETPIN, HIGH);  
}


// ============================= loop() ===================
void loop() 
{
  ReadSensors();                   // Sensor read to get top and bottom sensor values
#ifdef DEBUG 
  Serial.print("bottom sensor = ");
  Serial.print(bottomPhotoValue);
  Serial.print("   top sensor = ");
  Serial.print(topPhotoValue);
  Serial.print("   dir pin = ");
  Serial.print(directionVal);
  Serial.print("   previous value = ");
  Serial.println(previousValue);
#endif
  
  if (abs(bottomPhotoValue - topPhotoValue) > SENSORSENSITIVITY)
  {
    RotateStepper(STEPPERINCREMENT);
    delay(1000);
  }
  previousValue = topPhotoValue - bottomPhotoValue;
  if (previousValue < 0)                            // Wrong direction??
  {
    directionVal = !directionVal;
    while (previousValue < 0) {
      RotateStepper(STEPPERINCREMENT);
      ReadSensors();
      previousValue = topPhotoValue - bottomPhotoValue;
#ifdef DEBUG 
  Serial.print("***************** bottom sensor = ");
  Serial.print(bottomPhotoValue);
  Serial.print("   top sensor = ");
  Serial.print(topPhotoValue);
  Serial.print("   dir pin = ");
  Serial.print(directionVal);
  Serial.print("   previous value = ");
  Serial.println(previousValue);
#endif
    }                                  // End while (previousVal < 0)
    directionVal = !directionVal;
  }                                    // End if (previousVal < 0)
}

/*****
  This method rotates the stepper motor steps units, where a unit is
  defined by the type of stepper motor used. For a NEMA 17, each step
  corresponds to 1.8 degree, or 200 steps to make a single revolution.
  The Pololu motor driver is capable of microsteps less that 1.8 degrees,
  but this code does not make use of microstepping.
  
  Parameters:
    int steps    the number of steps we wish to rotate the motor
    
  Return value:
    void
*****/  
void RotateStepper(int steps)
{
  int i;
  
  digitalWrite(ENABLEPIN, LOW);            // LOW turns stepper driver on
  delay(SETTLESTEPPER);
  digitalWrite(DIRECTIONPIN, directionVal);
  delay(SETTLESTEPPER);
 
  for (i = 0; i < steps; i++) 
  {
    digitalWrite(STEPPIN, LOW);
    delay(SETTLESTEPPER);
    digitalWrite(STEPPIN, HIGH);
    delay(SETTLESTEPPER);
  }
} 

/*****
  This method reads the values from the two photoresistors that make up
  the sensor. It is assumed that the sensor is positioned correctly to
  the solar panel for reading.
  
  Parameters:
    void
    
  Return value:
    void
*****/ 
void ReadSensors()
{
  topPhotoValue = analogRead(TOPSENSORPIN); // Get values for top - bottom sensors
  delayMicroseconds(READSENSORDELAY);       // Prevent "digital jitters"
  bottomPhotoValue = analogRead(BOTTOMSENSORPIN);
  delayMicroseconds(READSENSORDELAY);   
}

